package com.barrelsofdata.examples.beam.streaming.sensor.function;

import com.barrelsofdata.examples.beam.streaming.sensor.model.Data;
import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.json.JsonMapper;
import org.apache.beam.sdk.transforms.SimpleFunction;
import org.apache.beam.sdk.values.KV;

import java.util.Objects;

/**
 * Parses kafka KV into {@link RawEvent}
 * Kafka key is used as id
 * Kafka value is used as {@link Data}
 * Throws {@link RuntimeException} on failed parsing
 */
public class ParseEvents extends SimpleFunction<KV<String, String>, RawEvent> {

    private JsonMapper jsonMapper;

    private ParseEvents(){}

    private ParseEvents(JsonMapper jsonMapper) {
        this.jsonMapper = jsonMapper;
    }

    @Override
    public RawEvent apply(KV<String, String> input) {
        try {
            String userId = Objects.requireNonNull(input).getKey();
            String eventDataJson = input.getValue();

            Data data = jsonMapper.readValue(eventDataJson, Data.class);
            return new RawEvent(userId, data);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    public static ParseEvents of(JsonMapper jsonMapper) {
        return new ParseEvents(jsonMapper);
    }
}
