package com.barrelsofdata.examples.beam.streaming.sensor.transform;

import com.barrelsofdata.examples.beam.streaming.sensor.function.ExtractTimestamp;
import com.barrelsofdata.examples.beam.streaming.sensor.function.ParseEvents;
import com.barrelsofdata.examples.beam.streaming.sensor.function.FailedEventBuilder;
import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import com.barrelsofdata.examples.beam.streaming.sensor.model.FailedEvent;
import com.barrelsofdata.examples.beam.streaming.sensor.config.Steps;
import com.fasterxml.jackson.databind.json.JsonMapper;
import org.apache.beam.sdk.transforms.MapElements;
import org.apache.beam.sdk.transforms.PTransform;
import org.apache.beam.sdk.transforms.WithFailures;
import org.apache.beam.sdk.transforms.WithTimestamps;
import org.apache.beam.sdk.values.KV;
import org.apache.beam.sdk.values.PCollection;
import org.apache.beam.sdk.values.TypeDescriptor;

/**
 * PTransform that converts key, value received from kafka into {@link RawEvent} object
 * and attaches the timestamp from event {@link com.barrelsofdata.examples.beam.streaming.sensor.model.Data}
 */
public class ParseWithTimestamp extends PTransform<PCollection<KV<String, String>>, PCollection<RawEvent>> {

    private final JsonMapper jsonMapper = new JsonMapper();

    @Override
    public PCollection<RawEvent> expand(PCollection<KV<String, String>> input) {
        WithFailures.Result<PCollection<RawEvent>, FailedEvent> parsedWithExceptions = input.apply(Steps.PARSE_EVENT.getStep(), MapElements
                .via(ParseEvents.of(jsonMapper))
                .exceptionsInto(TypeDescriptor.of(FailedEvent.class))
                .exceptionsVia(FailedEventBuilder.of(Steps.PARSE_EVENT.getStep())));

        PCollection<RawEvent> parsed = parsedWithExceptions.output();

        // Probably send the below to a dead letter kafka topic
        // PCollection<FailedEvent> failed = parsedWithExceptions.failures();

        return parsed.apply(Steps.ATTACH_TIMESTAMP.getStep(),
                WithTimestamps.of(ExtractTimestamp.of()));
    }
}
