package com.barrelsofdata.examples.beam.streaming.sensor.function;

import com.barrelsofdata.examples.beam.streaming.sensor.model.FailedEvent;
import org.apache.beam.sdk.transforms.WithFailures;
import org.apache.beam.sdk.values.KV;
import org.checkerframework.checker.initialization.qual.Initialized;
import org.checkerframework.checker.nullness.qual.NonNull;
import org.checkerframework.checker.nullness.qual.UnknownKeyFor;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class FailedEventBuilderTest {
    @Test
    void testPipelineException() {
        String step = "Test step";
        String exceptionMessage = "Test exception";

        KV<String, String> testInput = KV.of("testUser1", """
        {"ts":1698489173051,speed":50}""");
        WithFailures.ExceptionElement<KV<String, String>> testExceptionElement = new WithFailures.ExceptionElement<>() {
            @Override
            public KV<String, String> element() {
                return testInput;
            }
            @Override
            public @UnknownKeyFor @NonNull @Initialized Exception exception() {
                return new RuntimeException(exceptionMessage);
            }
        };

        FailedEventBuilder<KV<String, String>> failedEventBuilder = FailedEventBuilder.of(step);
        FailedEvent failedEvent = failedEventBuilder.apply(testExceptionElement);

        assertEquals(testInput.toString(), failedEvent.event());
        assertEquals(step, failedEvent.step());
        assertEquals(exceptionMessage, failedEvent.exception().getMessage());
    }

}