package com.barrelsofdata.examples.beam.streaming.sensor.function;

import com.barrelsofdata.examples.beam.streaming.sensor.model.Data;
import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import com.fasterxml.jackson.databind.json.JsonMapper;
import org.apache.beam.sdk.values.KV;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class ParseEventsTest {

    @Test
    void testValidEvent() {
        String id = "testId1";
        Long ts = 1698489173051L;
        Double value = 50.0;
        KV<String, String> testInput = KV.of(id, """
        {"ts":%s,"value":%s}""".formatted(ts, value));
        ParseEvents parseEvents = ParseEvents.of(new JsonMapper());
        RawEvent parsed = parseEvents.apply(testInput);
        Data parsedData = parsed.data();

        assertEquals(id, parsed.id());
        assertEquals(ts, parsedData.ts());
        assertEquals(value, parsedData.value());
    }

    @Test
    void testInvalidEvent() {
        KV<String, String> testInput = KV.of("testUser1", """
        {"ts":1698489173051,value":50}""");
        ParseEvents parseEvents = ParseEvents.of(new JsonMapper());

        assertThrows(RuntimeException.class, () -> parseEvents.apply(testInput));
    }

}